# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import logging
import os
from parallels.operability_check import DomainServicesOperabilityChecker
from parallels.operability_check import DomainDNSService, DNSRecord, DNSIssueType, Severity, DomainIssue
from utils import domain_idna_to_ascii
from OperabilityCheck import OperabilityCheck, OperabilityCheckResult, ConstraintViolation, RunError

_logger = logging.getLogger('operability_check.dns')

class OperabilityCheckDns(OperabilityCheck):
    def _validate(self):
        violations = []
        violations += self._validateDomainName(self._input.findtext('domain-name'), 'domain-name')
        for i, serverNode in enumerate(self._input.findall('dns-servers/dns-server')):
            violations += self._validateDnsServer(serverNode, '/'.join(['dns-servers/dns-server', str(i)]))
        for i, recordNode in enumerate(self._input.findall('dns-records/dns-record')):
            violations += self._validateDnsRecord(recordNode, '/'.join(['dns-records/dns-record', str(i)]))
        return violations

    def _run(self):
        domainName = self._input.findtext('domain-name')
        _logger.info(u'Check DNS-service of domain %s', domainName);

        try:
            domainAsciiName = domain_idna_to_ascii(domainName).encode('UTF-8')
            dnsRecords = []
            for recordNode in self._input.findall('dns-records/dns-record'):
                (rec_type, src, dst, opt) = (
                    recordNode.findtext('type')
                    , recordNode.findtext('src')
                    , recordNode.findtext('dst')
                    , recordNode.findtext('opt')
                )
                if 'PTR' == rec_type:
                    _logger.info(u'PTR records checking is not implemented yet, skip it');
                    continue
                if 'NS' == rec_type and 'nt' == os.name:
                    _logger.info(u'NS records checking is not implemented yet, skip it');
                    continue
                _logger.info(u'Check DNS-record (\'%s\', \'%s\', \'%s\', \'%s\')', rec_type, src, dst, opt)
                if rec_type in ['NS', 'MX', 'CNAME', 'PTR']:
                    dst = domain_idna_to_ascii(dst)
                dnsRecords.append(DNSRecord(rec_type, src, dst, opt))
            ipAddresses = []
            for serverNode in self._input.findall('dns-servers/dns-server'):
                ipAddress = serverNode.findtext('ip-address')
                _logger.info(u'Check DNS-records at \'%s\'', ipAddress)
                ipAddresses.append(ipAddress)
    
            checker = DomainServicesOperabilityChecker()
            issues = checker.check_dns([DomainDNSService(domainAsciiName, dnsRecords, ipAddresses)], dns_server_name='DNS server')
            _logger.info(u'Found %d issues', len(issues));
            return issues
        except Exception as e:
            _logger.debug(u"Run is failed:", exc_info=e)
            raise RunError(u'Unable to check the DNS-service of domain \'%s\': %s' % (domainName, unicode(e)))

    def _getResult(self, issue):
        result = self._results.get(issue.category, self._results[DNSIssueType.INTERNAL_ERROR])
        return OperabilityCheckResult(result['severity'], issue.problem, result['solution'])

    def _validateDnsServer(self, serverNode, path):
        return self._validateIpAddress(serverNode.findtext('ip-address'), '/'.join([path, 'ip-address']))

    def _validateDnsRecord(self, recordNode, path):
        violations = []
        violations += self._validateRequiredArgument(recordNode.findtext('type'), '/'.join([path, 'type']))
        return violations

    _results = {
        DNSIssueType.DIFFERENT_RECORDS: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The DNS zone of this domain exists in target Plesk and is enabled.
2. All desired DNS records exist in target Plesk and are active."""
        }
        , DNSIssueType.DNS_SERVER_TIMEOUT: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Make sure that:
1. The target Plesk server is up.
2. DNS server is started on the target Plesk server.
3. There are no firewall rules blocking requests to the server."""
        }
        , DNSIssueType.INTERNAL_ERROR: {
            'severity': Severity.INFO
            , 'solution': u"""\
Manually check whether this zone is resolved properly by the target Plesk DNS server."""
        }
    }

