# Copyright 1999-2015. Parallels IP Holdings GmbH. All Rights Reserved.
import logging
from parallels.operability_check import DomainServicesOperabilityChecker
from parallels.operability_check import DomainWebService, WebProtocol, WebsiteAvailabilityIssueType, Severity
from utils import domain_idna_to_ascii
from OperabilityCheck import OperabilityCheck, OperabilityCheckResult, RunError

_logger = logging.getLogger('operability_check.website_availability')

class OperabilityCheckWebsiteAvailability(OperabilityCheck):
    def _validate(self):
        violations = []
        violations += self._validateDomainName(self._input.findtext('domain-name'), 'domain-name')
        for i, serverNode in enumerate(self._input.findall('web-servers/web-server')):
            violations += self._validateWebServer(serverNode, '/'.join(['web-servers/web-server', str(i)]))
        return violations

    def _run(self):
        domainName = self._input.findtext('domain-name')
        _logger.info(u'Check web-service of domain %s', domainName);

        try:
            services = []
            domainAsciiName = domain_idna_to_ascii(domainName).encode('UTF-8')
            for serverNode in self._input.findall('web-servers/web-server'):
                service = DomainWebService(domainAsciiName, serverNode.findtext('ip-address'), serverNode.findtext('protocol'))
                _logger.info(u'Check web-service at \'%s://%s\'. Expected HTTP code: %s', service.protocol, service.web_server_ip, self._input.findtext('expected-http-code', ''))
                services.append(service)
    
            checker = DomainServicesOperabilityChecker()
            issues = checker.check_website_availability(services)
            _logger.info(u'Found %d issues', len(issues));
            return issues
        except Exception as e:
            _logger.debug(u"Run is failed:", exc_info=e)
            raise RunError(u'Unable to check the web-service of domain \'%s\': %s' % (domainName, unicode(e)))

    def _getResult(self, issue):
        if WebsiteAvailabilityIssueType.STATUS_CODE_5xx == issue.category and 500 <= int(self._input.findtext('expected-http-code', 200)):
            expectedHttpCode = self._input.findtext('expected-http-code')
            if None != expectedHttpCode and 500 <= int(expectedHttpCode):
                _logger.info(u'Web-service`s response code is in range 5XX, expected code is %d, skip issue. More precise checking is not implemented yet.', int(expectedHttpCode));
                return None
        result = self._results.get(issue.category, self._results[WebsiteAvailabilityIssueType.INTERNAL_ERROR])
        return OperabilityCheckResult(result['severity'], issue.problem, result['solution'])

    def _validateWebServer(self, serverNode, path):
        violations = []
        violations += self._validateIpAddress(serverNode.findtext('ip-address'), '/'.join([path, 'ip-address']))
        violations += self._validateEnum(serverNode.findtext('protocol'), '/'.join([path, 'protocol']), ['http', 'https'])
        return violations

    _results = {
        WebsiteAvailabilityIssueType.STATUS_CODE_5xx: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Please check that web server configuration is correct."""
        }
        , WebsiteAvailabilityIssueType.CONNECTION_ISSUE: {
            'severity': Severity.WARNING
            , 'solution': u"""\
Please check that web server is running and there are no firewall rules that block web service."""
        }
        , WebsiteAvailabilityIssueType.INTERNAL_ERROR: {
            'severity': Severity.INFO
            , 'solution': u"""\
Please perform web availability checks for the domain manually."""
        }
    }


